@@ -11,6 +11,10 @@
 #'   least 3.
 #' @param f A positive integer: the number of founders. Must be at least 2
 #'   unless selfing is allowed.
+#' @param maxDirectGap An integer; the maximum distance between direct
+#'   descendants allowed to mate. For example, the default value of 1 allows
+#'   parent-child mating, but not grandparent-grandchild. Set to `Inf` or `NULL`
+#'   to remove all restrictions.
 #' @param selfing A logical indicating if selfing is allowed. Default: FALSE.
 #' @param seed An integer seed for the random number generator (optional).
 #' @param g,founders Deprecated arguments.
@@ -18,11 +22,23 @@
 #' @return A connected pedigree returned as a `ped` object.
 #'
 #' @examples
-#' randomPed(8, f = 3, seed = 11)
-#' randomPed(8, f = 3, seed = 11, selfing = TRUE)
+#' plot(randomPed(7, f = 2, seed = 12))
+#'
+#' # Disallow mating between direct discendants
+#' plot(randomPed(7, f = 2, seed = 12, maxDirectGap = 0))
+#'
+#' # No restrictions on mating between direct discendants
+#' plot(randomPed(7, f = 2, seed = 12, maxDirectGap = Inf))
+#'
+#' # Allow selfing
+#' y = randomPed(5, f = 2, seed = 2, selfing = TRUE)
+#' hasSelfing(y)
+#' y
+#' plot(y, arrows = TRUE)
 #'
 #' @export
-randomPed = function(n, f = 2, selfing = FALSE, seed = NULL, g = NULL, founders = NULL) {
+randomPed = function(n, f = 2, maxDirectGap = 1, selfing = FALSE,
+                     seed = NULL, g = NULL, founders = NULL) {
   # TODO: Remove
   if(!is.null(g)) {
     message("Switching to the old version, using deprecated argument `g`.")
@@ -63,17 +79,24 @@ randomPed = function(n, f = 2, selfing = FALSE, seed = NULL, g = NULL, founders
   # Complete sex vector
   sex = c(sexFou, sexNonfou)
 
-  # List of components. Initially f singletons
+  # Component index.
+  # Initially 1,2,...f (singletons) and unknown (0) for the rest
   compvec = c(seq_len(f), rep(0L, n-f))
 
+  # Generation gap matrix
+  gengap = matrix(NA_integer_, nrow = n, ncol = n)
+
   for(k in seq(f + 1, to = n)) {
 
     sq = seq_len(k-1)
     ncomp = max(compvec)
     compSizes = tabulate(compvec)
 
-    # Surplus pairings
-    surplus = (n-k+1) - (ncomp-1)
+    # Minimum number of pairings required to make output connected
+    requ = ncomp-1
+
+    # Surplus pairings (i.e., leeway in addition to the required ones)
+    surplus = (n-k+1) - requ
 
     # Weights for choosing the first parent
     # The fewer surplus pairings, the more weight on small components.
@@ -82,12 +105,29 @@ randomPed = function(n, f = 2, selfing = FALSE, seed = NULL, g = NULL, founders
     w1 = wComp[compvec[sq]]
     w1 = w1/sum(w1)
 
+    # Candidates to avoid for first parent (maxDirectGap may be prohibitive)
+    if(!selfing && !is.null(maxDirectGap) && maxDirectGap < Inf) {
+       avoid = sapply(sq, function(i) {
+         gapsi = pmax(gengap[sq, i], gengap[i, sq], na.rm = TRUE)
+         toofar = !is.na(gapsi) & gapsi > maxDirectGap
+         samesex = sex[sq] == sex[i]
+         all(toofar | samesex)
+       })
+    }
+    else {
+      avoid = rep(FALSE, k-1)
+    }
+
     # Sample first parent!
-    par1 = sample.int(k-1, 1, prob = w1)
+    cand1 = sq[!avoid]
+    prob1 = w1[!avoid]
+    par1 = sample(cand1, size = 1, prob = prob1)
     comp1 = compvec[par1]
 
-    # Second parent: Any of opposite sex.
+    # Candidates for the other parent: Any of opposite sex
     isCand = sex[sq] != sex[par1]
+
+    # If selfing, include self as candidate
     if(selfing)
       isCand[par1] = TRUE
 
@@ -95,6 +135,19 @@ randomPed = function(n, f = 2, selfing = FALSE, seed = NULL, g = NULL, founders
     if(surplus == 0)
       isCand[compvec[sq] == comp1] = FALSE
 
+    # Apply generation gap limit if given
+    if(!is.null(maxDirectGap) && maxDirectGap < Inf) {
+      gaps = pmax(gengap[sq, par1], gengap[par1, sq], na.rm = TRUE)
+      toofar = !is.na(gaps) & gaps > maxDirectGap
+      isCand[toofar] = FALSE
+    }
+
+    # If no candidates, plot (for debugging)
+    if(!any(isCand)) {
+      p = ped(id[sq], fid[sq], mid[sq], sex[sq])
+      plot(p, hatched = par1)
+    }
+
     # The fewer surplus, the more probable to go to another comp
     par2 = safe_sample(sq[isCand], 1)
     comp2 = compvec[par2]
@@ -123,6 +176,13 @@ randomPed = function(n, f = 2, selfing = FALSE, seed = NULL, g = NULL, founders
       b = compvec > max(comp1, comp2)
       compvec[b] = compvec[b] - 1
     }
+
+    # Update generation gap matrix
+    gengap[sq, k] = 1L + pmax(gengap[sq, par1], gengap[sq, par2], na.rm = TRUE)
+    if(is.na(gengap[par1, k]))
+      gengap[par1, k] = 1L
+    if(is.na(gengap[par2, k]))
+      gengap[par2, k] = 1L
   }
 
   newPed(as.character(id), as.integer(fid), as.integer(mid), sex, FAMID = "")
